
var taskStatus = {
    "SUCCEEDED" : "bar",
    "FAILED" : "bar-failed",
    "RUNNING" : "bar-running",
    "KILLED" : "bar-killed"
};


tasks.sort(function(a, b) {
    return a.endDate - b.endDate;
});


tasks.sort(function(a, b) {
    return a.startDate - b.startDate;
});

//var maxDate = allTasks[allTasks.length - 1].endDate;
//var minDate = allTasks[0].startDate;

var listSize = 10;

var format = "%H:%M";
var timeDomainString = "1week";

var gantt = d3.gantt().taskTypes(taskNames).taskStatus(taskStatus).tickFormat(format);

var margin = {
     top : 20,
     right : 40,
     bottom : 20,
     left : 20
};
gantt.margin(margin);

gantt.timeDomainMode("fixed");
changeTimeDomain(timeDomainString);

gantt(tasks);

function formatDate(date){
	if (formatDate != null){
		return date.getFullYear()+"."+padDateEntry(date.getMonth())+"."+padDateEntry(date.getDate())+"  - "+ 
		padDateEntry(date.getHours())+":"+padDateEntry(date.getMinutes())+":"+padDateEntry(date.getSeconds())+"."+date.getMilliseconds();
	}
	return "";
}

function padDateEntry(entry){
	if (entry <10){
		return "0"+entry;
	}
	return entry;
}

function changeTimeDomain(timeDomainString) {
    this.timeDomainString = timeDomainString;
    switch (timeDomainString) {
    case "1hr":
	format = "%H:%M:%S";
	gantt.timeDomain([ d3.time.hour.offset(getEndDate(), -1+ timeOffset), d3.time.hour.offset(getEndDate(),  timeOffset)  ]);
	break;
    case "3hr":
	format = "%H:%M";
	gantt.timeDomain([ d3.time.hour.offset(getEndDate(), -3+ timeOffset*3),d3.time.hour.offset(getEndDate(),  timeOffset*3)  ]);
	break;

    case "6hr":
	format = "%H:%M";
	gantt.timeDomain([ d3.time.hour.offset(getEndDate(), -6 + timeOffset*6),d3.time.hour.offset(getEndDate(),  timeOffset*6) ]);
	break;
	
	case "12hr":
	format = "%H:%M";
	gantt.timeDomain([ d3.time.hour.offset(getEndDate(), -12 + timeOffset*12),d3.time.hour.offset(getEndDate(),  timeOffset*12) ]);
	break;

    case "1day":
	format = "%H:%M";
	gantt.timeDomain([ d3.time.day.offset(getEndDate(), -1+ timeOffset), d3.time.hour.offset(getEndDate(),  timeOffset)  ]);
	break;

    case "1week":
	format = "%a %H:%M";
	gantt.timeDomain([ d3.time.day.offset(getEndDate(), -7+ timeOffset), d3.time.hour.offset(getEndDate(),  timeOffset) ]);
	break;
    default:
	//format = "%H:%M"
	format = "%a %H:%M";
    }
    gantt.tickFormat(format);
    gantt.redraw(tasks);
}

function getEndDate() {
    var lastEndDate = Date.now();
    if (allTasks.length > 0) {
		lastEndDate = getMaxEndDate(allTasks);
    }

    return lastEndDate;
}


function refreshGantt(){
	gantt = d3.gantt().taskTypes(taskNames).taskStatus(taskStatus).tickFormat(format);
	gantt.margin(margin);
	gantt.timeDomainMode("fixed");

	changeTimeDomain(timeDomainString);
	gantt.redraw(tasks);
}

function shiftDown() {
	if (startPoz + listSize<allTaskNames.length){
		startPoz++; 
		document.getElementById("remainingUp").innerHTML = startPoz;
		document.getElementById("remainingDown").innerHTML =allTaskNames.length- startPoz-listSize;
	}else{
		alert("End of list");
		return;
	}
	taskNames = allTaskNames.slice(startPoz,startPoz+listSize);
	tasks = filterTasksByName(taskNames);
	refreshGantt();
	
};

function shiftUp() {
	if (startPoz >0){
		startPoz--;
		document.getElementById("remainingUp").innerHTML = startPoz;	
		document.getElementById("remainingDown").innerHTML = allTaskNames.length -startPoz - listSize;	
	}else{
		alert("End of list");
		return;
	}
	taskNames = allTaskNames.slice(startPoz,startPoz+listSize);	
	tasks = filterTasksByName(taskNames);
	refreshGantt();
};

function resetFilter() {
	taskNames = allTaskNames.slice(startPoz,startPoz+listSize);	
	tasks = filterTasksByName(taskNames);
	refreshGantt();
};

function showAllService(){
	taskNames = allTaskNames.slice();	
	tasks = filterTasksByName(taskNames);
	refreshGantt();
}

function showAllScheduledService(){
	taskNames = allScheduledTaskNames.slice();	
	tasks = filterTasksByName(taskNames);
	refreshGantt();
}

function filterTasksByName(filterTaskNames){
	var filteredTasks = [];
	for (i=0;i<allTasks.length;i++){
		for (j=0;j<filterTaskNames.length;j++){
			if (allTasks[i].taskName == filterTaskNames[j]){
				filteredTasks.push(allTasks[i])
			}
		}
	}	
	return filteredTasks;
}

function filterBySearchCriteria(){
	var searchValue = document.getElementById("serviceSeachValue");
	taskNames = findMatchingServices(searchValue.value);
	tasks = filterTasksByName(taskNames);	
	refreshGantt();	
}

function findMatchingServices(searchValue){
	var filteredTasks = [];
	for (i=0;i<allTaskNames.length;i++){
		if(allTaskNames[i].search(searchValue)!=-1){
			filteredTasks.push(allTaskNames[i]);
		}
	}
	//alert(filteredTasks);
	return filteredTasks;
}

function incrementTimeOffset(){
	timeOffset++;
    changeTimeDomain(timeDomainString);
    gantt.redraw(tasks);
}

function decrementTimeOffset(){
	timeOffset--;
    changeTimeDomain(timeDomainString);
    gantt.redraw(tasks);
}

function resetTimeOffset(){
	timeOffset = 0;
    changeTimeDomain(timeDomainString);
    gantt.redraw(tasks);
}

function showTaskDetails(taskIndex){	
	document.getElementById("serviceName").innerHTML = allTasks[taskIndex].taskName;
	document.getElementById("serviceUserID").innerHTML = allTasks[taskIndex].userID;
	document.getElementById("serviceStartDate").innerHTML = formatDate(allTasks[taskIndex].startDate);
	document.getElementById("serviceserverID").innerHTML = allTasks[taskIndex].serverID;
	document.getElementById("serviceDuration").innerHTML = allTasks[taskIndex].duration;
	document.getElementById("serviceError").innerHTML = allTasks[taskIndex].error;
	document.getElementById("allTasksSelect").value = allTasks[taskIndex].taskName;
	updateAllServiceCallsTable(allTasks[taskIndex].taskName);
}

function updateAllServiceCallsTable(selectedServiceName){
	var table = document.getElementById("serviceFullCallList");
	document.getElementById("serviceDetailsSeachValue").value = selectedServiceName;
	
	
	for (i=table.rows.length-1;i>0;i--){		
		table.deleteRow(i);
	}
	
	for (i=0;i<allTasks.length;i++){
		if (allTasks[i].taskName == selectedServiceName){
			var row = table.insertRow(1);
			row.style.background = "#BDB76B";
			var cell1 = row.insertCell(0);
			var cell2 = row.insertCell(1);
			var cell3 = row.insertCell(2);
			var cell4 = row.insertCell(3);
			var cell5 = row.insertCell(4);
			cell1.innerHTML = formatDate(allTasks[i].startDate);
			cell2.innerHTML = allTasks[i].duration;
			cell3.innerHTML = allTasks[i].userID;
			cell4.innerHTML = allTasks[i].serverID;
			cell5.innerHTML = allTasks[i].error;
		}
	}
}

function updateRemainingItemsDown(){
	document.getElementById("remainingDown").innerHTML = Math.max(0,allTaskNames.length- startPoz-listSize);
}

function getMaxEndDate(unsortedTasks){
	var allTasksSorted = unsortedTasks.slice().
	sort(function(a, b) {
			return a.endDate - b.endDate;
	});;
		
	return allTasksSorted[allTasksSorted.length - 1].endDate;
}

function getMinStartDate(unsortedTasks){
	var allTasksSorted = unsortedTasks.slice().
	sort(function(a, b) {
			return a.endDate - b.endDate;
	});;
		
	return allTasksSorted[0].endDate;
}
